'
' Checks if $1 item id is currently active.
' slot = item id of $1 item
' $1_item$ = RETURNed item
' RETURNs bOK: $$TRUE on success.
'
' Usage:
' IFF $1_Active ($1_id) THEN
'  PRINT "$1_Active: $1 item id "; $1_id; " is inactive"
' ENDIF
'
FUNCTION $1_Active ($1_id)
	SHARED $1_poolUM[]

	slot = $1_id - 1
	IF (slot >= 0) && (slot <= UBOUND ($1_poolUM[])) THEN
		IF $1_poolUM[slot] THEN RETURN $$TRUE
	ENDIF
END FUNCTION
'
' Finds the $1 item matching the passed search criterion.
' searchFor$ = value to search for
' RETURNs the found item id or 0 if not found.
'
' Usage:
'	$1_id = $1_Find (searchFor$)   ' find case insensitive
'	IF $1_id > 0 THEN
'		msg$ = "$1_Find: $1 item id " + STRING$ ($1_id) + " is an exact match for " + searchFor$
'		PRINT msg$
'	ENDIF
'
FUNCTION $1_Find (searchFor$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	$1_id = 0		' assume not found
	crit$ = TRIM$ (searchFor$)
	LEN_crit = LEN (crit$)

	SELECT CASE LEN_crit
		CASE 0		' error: no find criterion
		CASE ELSE
			IFZ $1_poolUM[] THEN EXIT SELECT
			FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
				IFF $1_poolUM[slot] THEN DO NEXT		' skip deleted spots
				try$ = TRIM$ ($1_pool$[slot])
				IF (LEN (try$) = LEN_crit) THEN
					IF try$ = crit$ THEN
						' $1_pool$[slot] is an exact match for searchFor$
						$1_id = slot + 1 : EXIT FOR
					ENDIF
				ENDIF
			NEXT slot
	END SELECT
	RETURN $1_id
END FUNCTION
'
' Finds case insensitive the $1 item matching
' the passed search criterion.
' searchFor$ = value to search for
' RETURNs the found item id or 0 if not found.
'
' Usage:
'	$1_id = $1_FindIns (searchFor$)   ' find case insensitive
'	IF $1_id > 0 THEN
'		msg$ = "$1_FindIns: $1 item id " + STRING$ ($1_id) + " is a case insensitive match for " + searchFor$
'		PRINT msg$
'	ENDIF
'
FUNCTION $1_FindIns (searchFor$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	$1_id = 0
	crit$ = TRIM$ (searchFor$)
	LEN_crit = LEN (crit$)

	SELECT CASE LEN_crit
		CASE 0
		CASE ELSE
			IFZ $1_poolUM[] THEN EXIT SELECT
			crit$ = UCASE$ (crit$)
			FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
				IFF $1_poolUM[slot] THEN DO NEXT		' skip deleted spots
				try$ = TRIM$ ($1_pool$[slot])
				IF (LEN (try$) = LEN_crit) THEN
					try$ = UCASE$ (try$)		' test case insensitive
					IF try$ = crit$ THEN
						$1_id = slot + 1 : EXIT FOR
					ENDIF
				ENDIF
			NEXT slot
	END SELECT
	RETURN $1_id
END FUNCTION
'
' Gets the the number of $1 items.
'
FUNCTION $1_Get_count ()
	SHARED $1_poolUM[]

	count = 0
	IF $1_poolUM[] THEN
		FOR slot = UBOUND ($1_poolUM[]) TO 0 STEP -1
			IF $1_poolUM[slot] THEN INC count
		NEXT slot
	ENDIF
	RETURN count
END FUNCTION
'
' Gets lower $1 item id.
'
' Usage:
'	$1_count = $1_Get_count ()
'	IF $1_count THEN
'		' *** walk thru the $1 pool
'		$1_idMin = $1_Get_idMin ()
'		$1_idMax = $1_Get_idMax ()
'		FOR $1_id = $1_idMin TO $1_idMax
'			bOK = $1_Get ($1_id, @$1_item$)
'			IFF bOK THEN DO NEXT		' deleted
'		NEXT $1_id
'	ENDIF
'
FUNCTION $1_Get_idMin ()
	SHARED $1_poolUM[]

	idMin = 0
	IF $1_poolUM[] THEN
		upper_slot = UBOUND ($1_poolUM[])
		FOR slot = 0 TO upper_slot
			IF $1_poolUM[slot] THEN
				idMin = slot + 1
				EXIT FOR
			ENDIF
		NEXT slot
	ENDIF
	RETURN idMin
END FUNCTION
'
' Gets next $1 item.
' $1_id = the id of the RETURNed item
' $1_item$ = RETURNed item
' RETURNs bOK: $$TRUE on success.
'
' Usage:
'	' *** walk thru the $1 pool
'	$1_id = 0
'	DO WHILE $1_GetNext (@$1_id, @$1_item$)
'	LOOP
'	IFZ $1_id THEN
'		PRINT "$1 pool is empty"
'	ENDIF
'
FUNCTION $1_GetNext (@$1_id, @$1_item$)
	SHARED $1_pool$[]
	SHARED $1_poolUM[]

	bOK = $$FALSE
	IF $1_poolUM[] THEN
		upper_slot = UBOUND ($1_poolUM[])
		FOR slot = $1_id TO upper_slot
			IF $1_poolUM[slot] THEN
				bOK = $$TRUE
				$1_id = slot + 1
				$1_item$ = $1_pool$[slot]
				EXIT FOR
			ENDIF
		NEXT slot
	ENDIF
	IFF bOK THEN
		$1_item$ = ""
	ENDIF
	RETURN bOK
END FUNCTION
'
' Extracts a quoted text from a string.
'
' text$     = the passed string
' pos1Quote = position of the first double-quote
'             < 1 indicates "first character"
' lit$      = RETURNed literal
'
' RETURNs bOK: $$TRUE if it has extracted the literal successfully.
'
FUNCTION $1_GetQuotedText (text$, pos1Quote, @lit$)

	bOK = $$FALSE

 	final = LEN (text$)		' 0 offset to null-terminator
	SELECT CASE final
		CASE 0		' the passed string is empty
			lit$ = ""
			'
		CASE ELSE
			IF (pos1Quote <= 0) THEN pos1Quote = 1
			'
			GOSUB double_quote
			'
	END SELECT

	RETURN bOK

SUB double_quote

	start = pos1Quote		' 1 offset to opening double-quote
	IF (text${pos1Quote - 1} != '"') THEN
		IF (start > 0) THEN DEC start		' not starting by a "
	ENDIF

	prevChar = 0 : cCh = 0
	FOR scans = start TO final
		IF (text${scans} = '"') THEN
			IF (prevChar != '\\') THEN
				IF (text${pos1Quote - 1} = '"') THEN bOK = $$TRUE		' OK!
				EXIT FOR
			ENDIF
		ENDIF
		INC cCh
		prevChar = text${scans}
	NEXT scans

	IFZ cCh THEN
		lit$ = ""
	ELSE
		lit$ = MID$(text$, (start + 1), cCh)
	ENDIF

END SUB

END FUNCTION